@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,26 +19,29 @@
 # a replica set. Thus each method asserts everything we want to assert for a
 # given replica-set configuration.
 
-import itertools
 import time
 import unittest
 
 import ha_tools
 from ha_tools import use_greenlets
 
-
-from pymongo.errors import AutoReconnect, OperationFailure, ConnectionFailure
-from pymongo.mongo_replica_set_client import Member, Monitor
+from nose.plugins.skip import SkipTest
+from pymongo.errors import (AutoReconnect,
+                            OperationFailure,
+                            ConnectionFailure,
+                            WTimeoutError)
+from pymongo.member import Member
+from pymongo.mongo_replica_set_client import Monitor
 from pymongo.mongo_replica_set_client import MongoReplicaSetClient
 from pymongo.mongo_client import MongoClient, _partition_node
 from pymongo.read_preferences import ReadPreference, modes
 
-from test import utils
+from test import utils, version
 from test.utils import one
 
 
-# Will be imported from time or gevent, below
-sleep = None
+# May be imported from gevent, below.
+sleep = time.sleep
 
 
 # Override default 30-second interval for faster testing
@@ -58,6 +61,33 @@ def partition_nodes(nodes):
     return [_partition_node(node) for node in nodes]
 
 
+# Backport permutations to Python 2.4.
+# http://docs.python.org/2.7/library/itertools.html#itertools.permutations
+def permutations(iterable, r=None):
+    pool = tuple(iterable)
+    n = len(pool)
+    if r is None:
+        r = n
+    if r > n:
+        return
+    indices = range(n)
+    cycles = range(n, n-r, -1)
+    yield tuple(pool[i] for i in indices[:r])
+    while n:
+        for i in reversed(range(r)):
+            cycles[i] -= 1
+            if cycles[i] == 0:
+                indices[i:] = indices[i+1:] + indices[i:i+1]
+                cycles[i] = n - i
+            else:
+                j = cycles[i]
+                indices[i], indices[-j] = indices[-j], indices[i]
+                yield tuple(pool[i] for i in indices[:r])
+                break
+        else:
+            return
+
+
 class HATestCase(unittest.TestCase):
     """A test case for connections to replica sets or mongos."""
 
@@ -370,12 +400,13 @@ class TestHealthMonitor(HATestCase):
             sleep(1)
             rs_state = c._MongoReplicaSetClient__rs_state
             if rs_state.writer and rs_state.writer != primary:
-                # New primary stepped up
-                new_primary = _partition_node(ha_tools.get_primary())
-                self.assertEqual(new_primary, rs_state.writer)
-                new_secondaries = partition_nodes(ha_tools.get_secondaries())
-                self.assertEqual(set(new_secondaries), rs_state.secondaries)
-                break
+                if ha_tools.get_primary():
+                    # New primary stepped up
+                    new_primary = _partition_node(ha_tools.get_primary())
+                    self.assertEqual(new_primary, rs_state.writer)
+                    new_secondaries = partition_nodes(ha_tools.get_secondaries())
+                    self.assertEqual(set(new_secondaries), rs_state.secondaries)
+                    break
         else:
             self.fail(
                 "No new primary after %s seconds. Old primary was %s, current"
@@ -388,6 +419,9 @@ class TestWritesWithFailover(HATestCase):
         res = ha_tools.start_replica_set([{}, {}, {}])
         self.seed, self.name = res
 
+        # Disable periodic refresh.
+        Monitor._refresh_interval = 1e6
+
     def test_writes_with_failover(self):
         c = MongoReplicaSetClient(
             self.seed, replicaSet=self.name, use_greenlets=use_greenlets)
@@ -398,21 +432,35 @@ class TestWritesWithFailover(HATestCase):
         db.test.insert({'foo': 'bar'}, w=w)
         self.assertEqual('bar', db.test.find_one()['foo'])
 
-        def try_write():
-            for _ in xrange(30):
-                try:
-                    db.test.insert({'bar': 'baz'})
-                    return True
-                except AutoReconnect:
-                    sleep(1)
-            return False
-
         killed = ha_tools.kill_primary(9)
         self.assertTrue(bool(len(killed)))
-        self.assertTrue(try_write())
+
+        # Wait past pool's check interval, so it throws an error from
+        # get_socket().
+        sleep(1)
+
+        # Verify that we only raise AutoReconnect, not some other error,
+        # while we wait for new primary.
+        for _ in xrange(10000):
+            try:
+                db.test.insert({'bar': 'baz'})
+
+                # No error, found primary.
+                break
+            except AutoReconnect:
+                sleep(.01)
+        else:
+            self.fail("Couldn't connect to new primary")
+
+        # Found new primary.
+        self.assertTrue(c.primary)
         self.assertTrue(primary != c.primary)
         self.assertEqual('baz', db.test.find_one({'bar': 'baz'})['bar'])
 
+    def tearDown(self):
+        Monitor._refresh_interval = MONITOR_INTERVAL
+        super(TestWritesWithFailover, self).tearDown()
+
 
 class TestReadWithFailover(HATestCase):
 
@@ -655,15 +703,11 @@ class TestReadPreference(HATestCase):
 
         # 3. PRIMARY UP, ONE SECONDARY DOWN -----------------------------------
         ha_tools.restart_members([killed])
-
-        for _ in range(30):
-            if ha_tools.get_primary():
-                break
-            sleep(1)
-        else:
-            self.fail("Primary didn't come back up")
+        ha_tools.wait_for_primary()
 
         ha_tools.kill_members([unpartition_node(secondary)], 2)
+        sleep(5)
+        ha_tools.wait_for_primary()
         self.assertTrue(MongoClient(
             unpartition_node(primary), use_greenlets=use_greenlets,
             read_preference=PRIMARY_PREFERRED
@@ -748,7 +792,7 @@ class TestReadPreference(HATestCase):
 
         # Verify that changing the mode unpins the member. We'll try it for
         # every relevant change of mode.
-        for mode0, mode1 in itertools.permutations(
+        for mode0, mode1 in permutations(
             (PRIMARY, SECONDARY, SECONDARY_PREFERRED, NEAREST), 2
         ):
             # Try reading and then changing modes and reading again, see if we
@@ -823,11 +867,12 @@ class TestReplicaSetAuth(HATestCase):
         self.db.logout()
         self.assertRaises(OperationFailure, self.db.foo.find_one)
 
-        primary = '%s:%d' % self.c.primary
-        ha_tools.kill_members([primary], 2)
+        primary = self.c.primary
+        ha_tools.kill_members(['%s:%d' % primary], 2)
 
         # Let monitor notice primary's gone
         sleep(2 * MONITOR_INTERVAL)
+        self.assertFalse(primary == self.c.primary)
 
         # Make sure we can still authenticate
         self.assertTrue(self.db.authenticate('user', 'userpass'))
@@ -943,9 +988,10 @@ class TestReplicaSetRequest(HATestCase):
 
         # Fail over
         ha_tools.kill_primary()
+        sleep(5)
+
         patience_seconds = 60
         for _ in range(patience_seconds):
-            sleep(1)
             try:
                 if ha_tools.ha_tools_debug:
                     print 'Waiting for failover'
@@ -954,6 +1000,8 @@ class TestReplicaSetRequest(HATestCase):
                     break
             except ConnectionFailure:
                 pass
+
+            sleep(1)
         else:
             self.fail("Problem with test: No new primary after %s seconds"
                 % patience_seconds)
@@ -976,6 +1024,146 @@ class TestReplicaSetRequest(HATestCase):
         super(TestReplicaSetRequest, self).tearDown()
 
 
+class TestLastErrorDefaults(HATestCase):
+
+    def setUp(self):
+        members = [{}, {}]
+        res = ha_tools.start_replica_set(members)
+        self.seed, self.name = res
+        self.c = MongoReplicaSetClient(self.seed, replicaSet=self.name,
+                                       use_greenlets=use_greenlets)
+
+    def test_get_last_error_defaults(self):
+        if not version.at_least(self.c, (1, 9, 0)):
+            raise SkipTest("Need MongoDB >= 1.9.0 to test getLastErrorDefaults")
+
+        replset = self.c.local.system.replset.find_one()
+        settings = replset.get('settings', {})
+        # This should cause a WTimeoutError for every write command
+        settings['getLastErrorDefaults'] = {
+            'w': 3,
+            'wtimeout': 1
+        }
+        replset['settings'] = settings
+        replset['version'] = replset.get("version", 1) + 1
+
+        self.c.admin.command("replSetReconfig", replset)
+
+        self.assertRaises(WTimeoutError, self.c.pymongo_test.test.insert,
+                          {'_id': 0})
+        self.assertRaises(WTimeoutError, self.c.pymongo_test.test.save,
+                          {'_id': 0, "a": 5})
+        self.assertRaises(WTimeoutError, self.c.pymongo_test.test.update,
+                          {'_id': 0}, {"$set": {"a": 10}})
+        self.assertRaises(WTimeoutError, self.c.pymongo_test.test.remove,
+                          {'_id': 0})
+
+    def tearDown(self):
+        self.c.close()
+        super(TestLastErrorDefaults, self).tearDown()
+
+
+class TestShipOfTheseus(HATestCase):
+    # If all of a replica set's members are replaced with new ones, is it still
+    # the same replica set, or a different one?
+    def setUp(self):
+        super(TestShipOfTheseus, self).setUp()
+        res = ha_tools.start_replica_set([{}, {}])
+        self.seed, self.name = res
+
+    def test_ship_of_theseus(self):
+        c = MongoReplicaSetClient(
+            self.seed, replicaSet=self.name, use_greenlets=use_greenlets)
+
+        db = c.pymongo_test
+        db.test.insert({}, w=len(c.secondaries) + 1)
+        find_one = db.test.find_one
+
+        primary = ha_tools.get_primary()
+        secondary1 = ha_tools.get_random_secondary()
+
+        new_hosts = []
+        for i in range(3):
+            new_hosts.append(ha_tools.add_member())
+
+            # RS closes all connections after reconfig.
+            for j in xrange(30):
+                try:
+                    if ha_tools.get_primary():
+                        break
+                except (ConnectionFailure, OperationFailure):
+                    pass
+
+                sleep(1)
+            else:
+                self.fail("Couldn't recover from reconfig")
+
+        # Wait for new members to join.
+        for _ in xrange(120):
+            if ha_tools.get_primary() and len(ha_tools.get_secondaries()) == 4:
+                break
+
+            sleep(1)
+        else:
+            self.fail("New secondaries didn't join")
+
+        ha_tools.kill_members([primary, secondary1], 9)
+        sleep(5)
+
+        # Wait for primary.
+        for _ in xrange(30):
+            if ha_tools.get_primary() and len(ha_tools.get_secondaries()) == 2:
+                break
+
+            sleep(1)
+        else:
+            self.fail("No failover")
+
+        sleep(2 * MONITOR_INTERVAL)
+
+        # No error.
+        find_one()
+        find_one(read_preference=SECONDARY)
+
+        # All members down.
+        ha_tools.kill_members(new_hosts, 9)
+        self.assertRaises(
+            ConnectionFailure,
+            find_one, read_preference=SECONDARY)
+
+        ha_tools.restart_members(new_hosts)
+
+        # Should be able to reconnect to set even though original seed
+        # list is useless. Use SECONDARY so we don't have to wait for
+        # the election, merely for the client to detect members are up.
+        sleep(2 * MONITOR_INTERVAL)
+        find_one(read_preference=SECONDARY)
+
+        # Kill new members and switch back to original two members.
+        ha_tools.kill_members(new_hosts, 9)
+        self.assertRaises(
+            ConnectionFailure,
+            find_one, read_preference=SECONDARY)
+
+        ha_tools.restart_members([primary, secondary1])
+
+        # Wait for members to figure out they're secondaries.
+        for _ in xrange(30):
+            try:
+                if len(ha_tools.get_secondaries()) == 2:
+                    break
+            except ConnectionFailure:
+                pass
+
+            sleep(1)
+        else:
+            self.fail("Original members didn't become secondaries")
+
+        # Should be able to reconnect to set again.
+        sleep(2 * MONITOR_INTERVAL)
+        find_one(read_preference=SECONDARY)
+
+
 if __name__ == '__main__':
     if use_greenlets:
         print('Using Gevent')
@@ -985,7 +1173,5 @@ if __name__ == '__main__':
         from gevent import monkey
         monkey.patch_socket()
         sleep = gevent.sleep
-    else:
-        sleep = time.sleep
 
     unittest.main()
