@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -34,13 +34,9 @@ from bson.py3compat import b, binary_type
 from bson.son import SON
 from nose.plugins.skip import SkipTest
 from test.test_client import get_client
-
+from pymongo.mongo_client import MongoClient
 
 class TestBinary(unittest.TestCase):
-
-    def setUp(self):
-        pass
-
     def test_binary(self):
         a_string = "hello world"
         a_binary = Binary(b("hello world"))
@@ -244,6 +240,14 @@ class TestBinary(unittest.TestCase):
             self.assertNotEqual(d['newguid'], d['newguidstring'])
         client.pymongo_test.drop_collection('csharp_uuid')
 
+    def test_uri_to_uuid(self):
+        if not should_test_uuid:
+            raise SkipTest("No uuid module")
+
+        uri = "mongodb://foo/?uuidrepresentation=csharpLegacy"
+        client = MongoClient(uri, _connect=False)
+        self.assertEqual(client.pymongo_test.test.uuid_subtype, CSHARP_LEGACY)
+
     def test_uuid_queries(self):
         if not should_test_uuid:
             raise SkipTest("No uuid module")
