@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,7 +19,6 @@ import unittest
 import sys
 sys.path[0:0] = [""]
 
-from bson.objectid import ObjectId
 from bson.son import SON
 from pymongo.database import Database
 from pymongo.son_manipulator import (NamespaceInjector,
@@ -35,6 +34,9 @@ class TestSONManipulator(unittest.TestCase):
     def setUp(self):
         self.db = Database(get_client(), "pymongo_test")
 
+    def tearDown(self):
+        self.db = None
+
     def test_basic(self):
         manip = SONManipulator()
         collection = self.db.test
