@@ -1,4 +1,4 @@
-# Copyright 2011-2012 10gen, Inc.
+# Copyright 2011-2014 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -28,6 +28,7 @@ from pymongo.uri_parser import (_partition,
                                 parse_uri)
 from pymongo.errors import ConfigurationError, InvalidURI
 from pymongo import ReadPreference
+from bson.binary import JAVA_LEGACY
 
 
 class TestURI(unittest.TestCase):
@@ -335,6 +336,50 @@ class TestURI(unittest.TestCase):
                          parse_uri("mongodb://user%40domain.com"
                                    "@localhost/foo?authMechanism=GSSAPI"))
 
+        res = copy.deepcopy(orig)
+        res['options'] = {'readpreference': ReadPreference.SECONDARY,
+                          'readpreferencetags': [
+                              {'dc': 'west', 'use': 'website'},
+                              {'dc': 'east', 'use': 'website'}]}
+        res['username'] = 'user@domain.com'
+        res['password'] = 'password'
+        res['database'] = 'foo'
+        self.assertEqual(res,
+                         parse_uri("mongodb://user%40domain.com:password"
+                                   "@localhost/foo?readpreference=secondary&"
+                                   "readpreferencetags=dc:west,use:website&"
+                                   "readpreferencetags=dc:east,use:website"))
+
+        res = copy.deepcopy(orig)
+        res['options'] = {'readpreference': ReadPreference.SECONDARY,
+                          'readpreferencetags': [
+                              {'dc': 'west', 'use': 'website'},
+                              {'dc': 'east', 'use': 'website'},
+                              {}]}
+        res['username'] = 'user@domain.com'
+        res['password'] = 'password'
+        res['database'] = 'foo'
+        self.assertEqual(res,
+                         parse_uri("mongodb://user%40domain.com:password"
+                                   "@localhost/foo?readpreference=secondary&"
+                                   "readpreferencetags=dc:west,use:website&"
+                                   "readpreferencetags=dc:east,use:website&"
+                                   "readpreferencetags="))
+
+        res = copy.deepcopy(orig)
+        res['options'] = {'uuidrepresentation': JAVA_LEGACY}
+        res['username'] = 'user@domain.com'
+        res['password'] = 'password'
+        res['database'] = 'foo'
+        self.assertEqual(res,
+                         parse_uri("mongodb://user%40domain.com:password"
+                                   "@localhost/foo?uuidrepresentation="
+                                   "javaLegacy"))
+
+        self.assertRaises(ConfigurationError, parse_uri,
+                          "mongodb://user%40domain.com:password"
+                          "@localhost/foo?uuidrepresentation=notAnOption")
+
     def test_parse_uri_unicode(self):
         # Ensure parsing a unicode returns option names that can be passed
         # as kwargs. In Python 2.4, keyword argument names must be ASCII.
